--- old/usr/src/uts/common/io/scsi/adapters/mpt_sas/mptsas.c	2014-12-19 13:49:23.718994526 -0500
+++ new/usr/src/uts/common/io/scsi/adapters/mpt_sas/mptsas.c	2014-12-19 13:49:23.729999960 -0500
@@ -1063,6 +1063,7 @@
 	char			smp_attach_setup = 0;
 	char			mutex_init_done = 0;
 	char			event_taskq_create = 0;
+        char			reset_taskq_create = 0;
 	char			dr_taskq_create = 0;
 	char			doneq_thread_create = 0;
 	char			added_watchdog = 0;
@@ -1269,6 +1270,18 @@
 	}
 	dr_taskq_create++;
 
+	/*
+	 * A taskq is created for dealing with reset events
+	 */
+	if ((mpt->m_reset_taskq = ddi_taskq_create(dip,
+	    "mptsas_reset_taskq",
+	    1, TASKQ_DEFAULTPRI, 0)) == NULL) {
+		mptsas_log(mpt, CE_NOTE, "ddi_taskq_create for reset "
+		    "failed");
+		goto fail;
+	}
+	reset_taskq_create++;
+
 	mpt->m_doneq_thread_threshold = ddi_prop_get_int(DDI_DEV_T_ANY, dip,
 	    0, "mptsas_doneq_thread_threshold_prop", 10);
 	mpt->m_doneq_length_threshold = ddi_prop_get_int(DDI_DEV_T_ANY, dip,
@@ -1593,6 +1606,9 @@
 		if (dr_taskq_create) {
 			ddi_taskq_destroy(mpt->m_dr_taskq);
 		}
+		if (reset_taskq_create) {
+			ddi_taskq_destroy(mpt->m_reset_taskq);
+		}
 		if (mutex_init_done) {
 			mutex_destroy(&mpt->m_tx_waitq_mutex);
 			mutex_destroy(&mpt->m_passthru_mutex);
@@ -1725,6 +1741,7 @@
 	mutex_exit(&mpt->m_mutex);
 
 	/* drain the taskq */
+	ddi_taskq_wait(mpt->m_reset_taskq);
 	ddi_taskq_wait(mpt->m_event_taskq);
 	ddi_taskq_wait(mpt->m_dr_taskq);
 
@@ -1900,6 +1917,7 @@
 	(void) mptsas_ioc_reset(mpt, FALSE);
 	mutex_exit(&mpt->m_mutex);
 	mptsas_rem_intrs(mpt);
+	ddi_taskq_destroy(mpt->m_reset_taskq);
 	ddi_taskq_destroy(mpt->m_event_taskq);
 	ddi_taskq_destroy(mpt->m_dr_taskq);
 
@@ -12737,6 +12755,12 @@
 	return (status);
 }
 
+/* Dirty wrapper for taskq */
+void
+mptsas_handle_restart_ioc(void *mpt) {
+    mptsas_restart_ioc((mptsas_t *) mpt);
+}
+
 int
 mptsas_restart_ioc(mptsas_t *mpt)
 {
